@@ -39,12 +39,12 @@ fn token_steam_string(arg: proc_macro2::TokenStream) -> String {
 }
 
 
-fn convert_to_arg_access(context: &str, arg: Expr, as_proxy: bool, ignore: &[&str]) -> Expr {
+fn convert_to_arg_access(context: &str, arg: Expr, as_proxy: bool, ignore: &[String]) -> Expr {
     match arg {
         Expr::Path(b) => {
             let token = b.to_token_stream().to_string();
             if token == "null" {
-                return syn::parse_str::<Expr>("serde_json::Value::Null").unwrap();
+                return syn::parse_str::<Expr>("bson2::Bson::Null").unwrap();
             }
             if token == "sql" {
                 return Expr::Path(b);
@@ -63,7 +63,7 @@ fn convert_to_arg_access(context: &str, arg: Expr, as_proxy: bool, ignore: &[&st
                 }
             }
             if fetch_from_arg {
-                return syn::parse_str::<Expr>(&format!(" &arg[\"{}\"]", param)).unwrap();
+                return syn::parse_str::<Expr>(&format!(" arg.index(\"{}\")", param)).unwrap();
             } else {
                 return syn::parse_str::<Expr>(&format!("{}", param)).unwrap();
             }
@@ -194,45 +194,13 @@ fn convert_to_arg_access(context: &str, arg: Expr, as_proxy: bool, ignore: &[&st
         }
         Expr::Field(mut b) => {
             b.base = Box::new(convert_to_arg_access(context, *b.base.clone(), as_proxy, ignore));
-            return match b.member.clone() {
-                Member::Named(n) => {
-                    let s = b.member.to_token_stream().to_string();
-                    let vs: Vec<&str> = s.split(".").collect();
-                    let mut token = String::new();
-                    for x in vs {
-                        if x.ends_with("()") {
-                            token.push_str(".");
-                            token.push_str(x.trim());
-                        } else {
-                            let x = x.trim();
-                            //format index
-                            let xs: Vec<&str> = x.split("[").collect();
-                            if xs.len() > 1 {
-                                let mut is_first = true;
-                                for x in xs {
-                                    if is_first {
-                                        token.push_str("[\"");
-                                        token.push_str(x.trim());
-                                        token.push_str("\"]");
-                                    } else {
-                                        token.push_str("[");
-                                        token.push_str(x.trim());
-                                    }
-                                    is_first = false;
-                                }
-                            } else {
-                                token.push_str("[\"");
-                                token.push_str(x.trim());
-                                token.push_str("\"]");
-                            }
-                        }
-                    }
-                    syn::parse_str::<Expr>(&format!("{}{}", b.base.to_token_stream(), token)).unwrap()
+            match b.member{
+                Member::Named(named) => {
+                    return syn::parse_str::<Expr>(&format!("{}.index(\"{}\")",b.base.to_token_stream(),named.to_token_stream())).unwrap();
                 }
-                Member::Unnamed(unamed) => {
-                    Expr::Field(b)
-                }
-            };
+                Member::Unnamed(_) => {}
+            }
+            return Expr::Field(b);
         }
         Expr::Reference(mut b) => {
             b.expr = Box::new(convert_to_arg_access(context, *b.expr, as_proxy, ignore));
@@ -241,8 +209,7 @@ fn convert_to_arg_access(context: &str, arg: Expr, as_proxy: bool, ignore: &[&st
         }
         Expr::Index(mut b) => {
             b.expr = Box::new(convert_to_arg_access(context, *b.expr, as_proxy, ignore));
-            let result = Expr::Index(b);
-            return syn::parse_str::<Expr>(&format!("{}", result.to_token_stream().to_string())).unwrap();
+            return syn::parse_str::<Expr>(&format!("{}.index({})",b.expr.to_token_stream(),b.index.to_token_stream())).unwrap();
         }
         Expr::Let(mut let_expr) => {
             let_expr.expr = Box::new(convert_to_arg_access(context, *let_expr.expr, as_proxy, ignore));
@@ -321,8 +288,8 @@ fn expr_type(expr: Expr) -> String {
     }
 }
 
-
-pub fn impl_fn(context: &str, func_name_ident: &str, args: &str, serialize_result: bool, as_proxy: bool, ignore: &[&str]) -> proc_macro2::TokenStream {
+/// gen method or body(if func_name_ident is empty)
+pub fn impl_fn(context: &str, func_name_ident: &str, args: &str, serialize_result: bool, as_proxy: bool, ignore: &[String]) -> proc_macro2::TokenStream {
     let mut string_data = args.to_string();
     string_data = string_data[1..string_data.len() - 1].to_string();
     string_data = string_data.replace(".string()", ".to_string()");
@@ -353,7 +320,7 @@ pub fn impl_fn(context: &str, func_name_ident: &str, args: &str, serialize_resul
     let t = t.unwrap();
     let mut result_impl = quote! { result };
     if serialize_result {
-        result_impl = quote! {serde_json::json!(result)};
+        result_impl = quote! {bson2::bson!(result)};
     }
     if func_name_ident.is_empty() || func_name_ident.eq("\"\"") {
         return quote! {
@@ -365,7 +332,7 @@ pub fn impl_fn(context: &str, func_name_ident: &str, args: &str, serialize_resul
     } else {
         let func_name_ident = Ident::new(&func_name_ident.to_string(), Span::call_site());
         return quote! {
-        pub fn #func_name_ident(arg:&serde_json::Value) -> serde_json::Value {
+        pub fn #func_name_ident(arg:&bson2::Bson) -> bson2::Bson {
            let result={#t};
            #result_impl
         }
