@@ -15,7 +15,7 @@ use crate::proc_macro::TokenStream;
 use crate::py_sql::{NodeType, ParsePySql};
 use crate::string_util::find_convert_string;
 
-fn parse_html_str(html: &str, format_char: char, fn_name: &str, ignore: &[&str]) -> proc_macro2::TokenStream {
+fn parse_html_str(html: &str, format_char: char, fn_name: &str, ignore: &mut Vec<String>) -> proc_macro2::TokenStream {
     let datas = load_html(html).expect("load_html() fail!");
     let mut sql_map = HashMap::new();
     let datas = include_replace(datas, &mut sql_map);
@@ -127,7 +127,7 @@ fn include_replace(htmls: Vec<Element>, sql_map: &mut HashMap<String, Vec<Elemen
     return results;
 }
 
-fn parse_html_node(htmls: Vec<Element>, format_char: char, ignore: &[&str]) -> proc_macro2::TokenStream {
+fn parse_html_node(htmls: Vec<Element>, format_char: char, ignore: &mut Vec<String>) -> proc_macro2::TokenStream {
     #[cfg(feature = "debug_mode")]
         {
             println!("load html:{:#?}", htmls);
@@ -154,7 +154,7 @@ fn to_mod(m: &ItemMod, t: &proc_macro2::TokenStream) -> TokenStream {
 
 
 /// gen rust code
-fn parse(arg: &Vec<Element>, methods: &mut proc_macro2::TokenStream, block_name: &str, format_char: char, ignore: &[&str]) -> proc_macro2::TokenStream {
+fn parse(arg: &Vec<Element>, methods: &mut proc_macro2::TokenStream, block_name: &str, format_char: char, ignore: &mut Vec<String>) -> proc_macro2::TokenStream {
     let mut body = quote! {};
     let fix_sql = quote! {
         rbatis_sql::sql_index!(sql,#format_char);
@@ -174,29 +174,24 @@ fn parse(arg: &Vec<Element>, methods: &mut proc_macro2::TokenStream, block_name:
                 impl_println(x, &mut body, ignore);
             }
             "" => {
-                let mut string_data = x.data.trim().to_string();
+                let mut string_data = x.data.to_string();
                 let convert_list = find_convert_string(&string_data);
                 let mut replaces = quote! {};
 
                 let mut replaced = HashMap::<String, bool>::new();
                 for (k, v) in convert_list {
                     let (method_name_string, method_name) = gen_method_name(&format!("{}:{}", block_name, k));
-                    let method_impl = crate::func::impl_fn(&body.to_string(), &method_name.to_string(), &format!("\"{}\"", k), false, true, ignore);
-                    let mut method_string = method_impl.to_string();
-                    let method_impl = method_string[method_string.find("{").unwrap()..method_string.len()].to_string();
-                    let method_impl = parse_expr(&method_impl);
+                    let method_impl = crate::func::impl_fn(&body.to_string(), "", &format!("\"{}\"", k), false, true, ignore);
                     //check append value
-                    if !body.to_string().contains(&format!("{} ", method_name)) {
-                        body = quote! {
+                    body = quote! {
                               #body
                               let #method_name = #method_impl;
                           };
-                    }
                     if v.starts_with("#") {
                         string_data = string_data.replacen(&v, &"?", 1);
                         body = quote! {
                               #body
-                              args.push(serde_json::json!(#method_name));
+                              args.push(bson2::bson!(#method_name));
                           };
                     } else {
                         if replaced.get(&v).is_none() {
@@ -218,7 +213,12 @@ fn parse(arg: &Vec<Element>, methods: &mut proc_macro2::TokenStream, block_name:
                 }
             }
             "if" => {
-                impl_if(x, &mut body, methods, quote! {}, &format!("{}:{}", block_name, "if"), format_char, ignore);
+                let test_value = x.attributes.get("test").expect(&format!("{} element must be have test field!", x.tag));
+                let mut if_tag_body = quote! {};
+                if x.childs.len() != 0 {
+                    if_tag_body = parse(&x.childs, methods, block_name, format_char, ignore);
+                }
+                impl_if(test_value, if_tag_body, &mut body, methods, quote! {}, &format!("{}:{}", block_name, "if"), format_char, ignore);
             }
             "trim" => {
                 let empty_string = String::new();
@@ -236,32 +236,23 @@ fn parse(arg: &Vec<Element>, methods: &mut proc_macro2::TokenStream, block_name:
                 let value = x.attributes.get("value").expect("<bind> element must be have value!").to_string();
 
                 let name_expr = parse_expr(&name);
-
-                let method_impl = crate::func::impl_fn(&body.to_string(), "this_is_gen", &format!("\"{}\"", value), false, true, ignore);
-
-                let method_string = method_impl.to_string();
-                let method_impl = &method_string[method_string.find("{").unwrap()..method_string.len()];
-                let method_impl = parse_expr(&method_impl);
-
+                let method_impl = crate::func::impl_fn(&body.to_string(), "", &format!("\"{}\"", value), false, true, ignore);
                 body = quote! {
                             #body
-                            arg[#name] = serde_json::Value::from(#method_impl);
+                            //bind
+                            let #name_expr = bson2::bson!(#method_impl);
                         };
-                //re gen new ident get method!
-                let (method_name_string, method_name) = gen_method_name(&format!("{}:{}", block_name, name));
-                let method_impl = crate::func::impl_fn(&body.to_string(), &method_name.to_string(), &format!("\"{}\"", name), false, true, ignore);
-                let mut method_string = method_impl.to_string();
-                let method_impl = method_string[method_string.find("{").unwrap()..method_string.len()].to_string();
-                let method_impl = parse_expr(&method_impl);
-                body = quote! {
-                            #body
-                            let #method_name = #method_impl;
-                        };
-
+                ignore.push(name);
             }
 
             "where" => {
-                impl_trim("", " ", "and |or ", " and| or", x, &mut body, arg, methods, &format!("{}:{}", block_name, "where:trim"), format_char, ignore);
+                impl_trim(" where ", " ", " |and |or ", " | and| or", x, &mut body, arg, methods, &format!("{}:{}", block_name, "where:trim"), format_char, ignore);
+                body = quote! {
+                            #body
+                            //check ends with where
+                            sql = sql.trim_end().to_string();
+                            sql = sql.trim_end_matches(" where").to_string();
+                        };
             }
 
             "choose" => {
@@ -271,22 +262,27 @@ fn parse(arg: &Vec<Element>, methods: &mut proc_macro2::TokenStream, block_name:
                         panic!("choose node's childs must be when node and otherwise node!");
                     }
                     if x.tag.eq("when") {
-                        impl_if(x, &mut inner_body, methods, quote! {return sql;}, &format!("{}:{}", block_name, "choose:when:if"), format_char, ignore);
+                        let test_value = x.attributes.get("test").expect(&format!("{} element must be have test field!", x.tag));
+                        let mut if_tag_body = quote! {};
+                        if x.childs.len() != 0 {
+                            if_tag_body = parse(&x.childs, methods, block_name, format_char, ignore);
+                        }
+                        impl_if(test_value, if_tag_body, &mut inner_body, methods, quote! {return sql;}, &format!("{}:{}", block_name, "choose:when:if"), format_char, ignore);
                     }
                     if x.tag.eq("otherwise") {
-                        impl_otherwise(x, &mut inner_body, methods, &format!("{}:{}", block_name, "choose:otherwise"), format_char, ignore);
+                        let child_body = parse(&x.childs, methods, block_name, format_char, ignore);
+                        impl_otherwise(child_body, &mut inner_body, methods, &format!("{}:{}", block_name, "choose:otherwise"), format_char, ignore);
                     }
                 }
                 body = quote! {
                 #body
-                let do_choose=||->String{
-                   let mut sql = String::new();
-                   #inner_body
-                   return sql;
+                let mut do_choose = || -> String {
+                         let mut sql = String::new();
+                         #inner_body
+                         return sql;
                 };
-                let choose_strings = do_choose();
                 sql.push_str(" ");
-                sql.push_str(&choose_strings);
+                sql.push_str(&do_choose());
                 sql.push_str(" ");
               }
             }
@@ -298,27 +294,30 @@ fn parse(arg: &Vec<Element>, methods: &mut proc_macro2::TokenStream, block_name:
                 let def_index = "index".to_string();
 
                 let collection = x.attributes.get("collection").unwrap_or(&empty_string).to_string();
-                let item = x.attributes.get("item").unwrap_or(&def_item).to_string();
-                let findex = x.attributes.get("index").unwrap_or(&def_index).to_string();
+                let mut item = x.attributes.get("item").unwrap_or(&def_item).to_string();
+                let mut findex = x.attributes.get("index").unwrap_or(&def_index).to_string();
                 let open = x.attributes.get("open").unwrap_or(&empty_string).to_string();
                 let close = x.attributes.get("close").unwrap_or(&empty_string).to_string();
                 let separator = x.attributes.get("separator").unwrap_or(&empty_string).to_string();
 
-                let impl_body = parse(&x.childs, methods, "foreach", format_char, &[findex.as_str(), item.as_str()]);
-                let (method_name_string, method_name) = gen_method_name(&format!("{}:{}", block_name, collection));
-                let method_impl = crate::func::impl_fn(&body.to_string(), &method_name.to_string(), &format!("\"{}\"", collection), false, false, ignore);
-                let method_string = method_impl.to_string();
-                let method_impl = method_string[method_string.find("{").unwrap()..method_string.len()].to_string();
+                if item.is_empty(){
+                    item = def_item;
+                }
+                if findex.is_empty(){
+                    findex = def_index;
+                }
+                let mut ignores = ignore.clone();
+                ignores.push(findex.to_string());
+                ignores.push(item.to_string());
 
-                let method_impl = parse_expr(&method_impl);
+                let impl_body = parse(&x.childs, methods, "foreach", format_char, &mut ignores);
+                let (method_name_string, method_name) = gen_method_name(&format!("{}:{}", block_name, collection));
+                let method_impl = crate::func::impl_fn(&body.to_string(), "", &format!("\"{}\"", collection), false, false, ignore);
                 //check append value
-                if !body.to_string().contains(&format!("{} ", method_name)) {
-                    body = quote! {
+                body = quote! {
                               #body
                               let #method_name = #method_impl;
                           };
-                }
-
                 body = quote! {
                     #body
                     sql.push_str(" ");
@@ -335,8 +334,6 @@ fn parse(arg: &Vec<Element>, methods: &mut proc_macro2::TokenStream, block_name:
                     sql.push_str(#close);
                     };
                 }
-
-
                 let item_ident = Ident::new(&item, Span::call_site());
                 let index_ident = Ident::new(&findex, Span::call_site());
 
@@ -366,7 +363,7 @@ fn parse(arg: &Vec<Element>, methods: &mut proc_macro2::TokenStream, block_name:
                         #open_impl
                         #index_create
                         {
-                          let foreach_arr = #method_name.as_array().unwrap();
+                          let foreach_arr = #method_name.array().unwrap();
                           for #item_ident in foreach_arr {
                             let item=#item_ident;
                             #impl_body
@@ -379,7 +376,7 @@ fn parse(arg: &Vec<Element>, methods: &mut proc_macro2::TokenStream, block_name:
                     }else if #method_name.is_object(){
                         #open_impl
                         {
-                          let foreach_arr = #method_name.as_object().unwrap();
+                          let foreach_arr = #method_name.object().unwrap();
                           for (#index_ident,#item_ident) in foreach_arr {
                               let item=#item_ident;
                               #impl_body
@@ -397,7 +394,7 @@ fn parse(arg: &Vec<Element>, methods: &mut proc_macro2::TokenStream, block_name:
             }
 
             "set" => {
-                impl_trim(" set ", " ", ",", ",", x, &mut body, arg, methods, &format!("{}:{}", block_name, "set:trim"), format_char, ignore);
+                impl_trim(" set ", " ", " |,", " |,", x, &mut body, arg, methods, &format!("{}:{}", block_name, "set:trim"), format_char, ignore);
             }
 
             "select" => {
@@ -405,7 +402,7 @@ fn parse(arg: &Vec<Element>, methods: &mut proc_macro2::TokenStream, block_name:
                 let method_name = Ident::new(id, Span::call_site());
                 let child_body = parse(&x.childs, methods, "select", format_char, ignore);
                 let select = quote! {
-                            pub fn #method_name (arg:&mut serde_json::Value) -> (String,Vec<serde_json::Value>) {
+                            pub fn #method_name (arg:&bson2::Bson) -> (String,Vec<bson2::Bson>) {
                                use rbatis_sql::ops::AsProxy;
                                let mut sql = String::with_capacity(1000);
                                let mut args = Vec::with_capacity(20);
@@ -424,7 +421,7 @@ fn parse(arg: &Vec<Element>, methods: &mut proc_macro2::TokenStream, block_name:
                 let method_name = Ident::new(id, Span::call_site());
                 let child_body = parse(&x.childs, methods, "select", format_char, ignore);
                 let select = quote! {
-                            pub fn #method_name (arg:&mut serde_json::Value) -> (String,Vec<serde_json::Value>) {
+                            pub fn #method_name (arg:&bson2::Bson) -> (String,Vec<bson2::Bson>) {
                                use rbatis_sql::ops::AsProxy;
 
                                let mut sql = String::with_capacity(1000);
@@ -444,7 +441,7 @@ fn parse(arg: &Vec<Element>, methods: &mut proc_macro2::TokenStream, block_name:
                 let method_name = Ident::new(id, Span::call_site());
                 let child_body = parse(&x.childs, methods, "select", format_char, ignore);
                 let select = quote! {
-                            pub fn #method_name (arg:&mut serde_json::Value) -> (String,Vec<serde_json::Value>) {
+                            pub fn #method_name (arg:&bson2::Bson) -> (String,Vec<bson2::Bson>) {
                                use rbatis_sql::ops::AsProxy;
 
                                let mut sql = String::with_capacity(1000);
@@ -464,7 +461,7 @@ fn parse(arg: &Vec<Element>, methods: &mut proc_macro2::TokenStream, block_name:
                 let method_name = Ident::new(id, Span::call_site());
                 let child_body = parse(&x.childs, methods, "select", format_char, ignore);
                 let select = quote! {
-                            pub fn #method_name (arg:&mut serde_json::Value) -> (String,Vec<serde_json::Value>) {
+                            pub fn #method_name (arg:&bson2::Bson) -> (String,Vec<bson2::Bson>) {
                                use rbatis_sql::ops::AsProxy;
                                
                                let mut sql = String::with_capacity(1000);
@@ -488,7 +485,7 @@ fn parse(arg: &Vec<Element>, methods: &mut proc_macro2::TokenStream, block_name:
 }
 
 
-fn impl_println(x: &Element, body: &mut proc_macro2::TokenStream, ignore: &[&str]) {
+fn impl_println(x: &Element, body: &mut proc_macro2::TokenStream, ignore: &mut Vec<String>) {
     let value = x.attributes.get("value").expect(&format!("{} element must be have value field!", x.tag));
     let method_name = impl_method(value, body, ignore);
     let mut format = String::new();
@@ -518,29 +515,20 @@ fn gen_method_name(test_value: &str) -> (String, Ident) {
 }
 
 
-fn impl_method(test_value: &str, body: &mut proc_macro2::TokenStream, ignore: &[&str]) -> Ident {
+fn impl_method(test_value: &str, body: &mut proc_macro2::TokenStream, ignore: &mut Vec<String>) -> Ident {
     let (_, method_name) = gen_method_name(&test_value);
-    let method_impl = crate::func::impl_fn(&body.to_string(), &method_name.to_string(), &format!("\"{}\"", test_value), false, true, ignore);
-    let method_string = method_impl.to_string();
-    let method_impl = &method_string[method_string.find("{").unwrap()..method_string.len()];
-    let s = syn::parse::<syn::LitStr>(method_impl.to_token_stream().into()).unwrap();
-    let method_impl = syn::parse_str::<Expr>(&s.value()).unwrap();
+    let method_impl = crate::func::impl_fn(&body.to_string(), "", &format!("\"{}\"", test_value), false, true, ignore);
     //check append value
-    if !body.to_string().contains(&format!("{} ", method_name)) {
-        *body = quote! {
-                              #body
-                              let #method_name = #method_impl;
-                          };
-    }
+    *body = quote! {
+                 #body
+               let #method_name = #method_impl;
+    };
     return method_name;
 }
 
-fn impl_if(x: &Element, body: &mut proc_macro2::TokenStream, methods: &mut proc_macro2::TokenStream, appends: proc_macro2::TokenStream, block_name: &str, format_char: char, ignore: &[&str]) {
-    let test_value = x.attributes.get("test").expect(&format!("{} element must be have test field!", x.tag));
+fn impl_if(test_value: &str, if_tag_body: proc_macro2::TokenStream, body: &mut proc_macro2::TokenStream, methods: &mut proc_macro2::TokenStream, appends: proc_macro2::TokenStream, block_name: &str, format_char: char, ignore: &mut Vec<String>) {
     let method_name = impl_method(test_value, body, ignore);
-    if x.childs.len() != 0 {
-        let if_tag_body = parse(&x.childs, methods, block_name, format_char, ignore);
-        *body = quote! {
+    *body = quote! {
                               #body
                               if #method_name {
                                    sql.push_str(" ");
@@ -549,11 +537,9 @@ fn impl_if(x: &Element, body: &mut proc_macro2::TokenStream, methods: &mut proc_
                                    #appends
                               }
                           };
-    }
 }
 
-fn impl_otherwise(x: &Element, body: &mut proc_macro2::TokenStream, methods: &mut proc_macro2::TokenStream, block_name: &str, format_char: char, ignore: &[&str]) {
-    let child_body = parse(&x.childs, methods, block_name, format_char, ignore);
+fn impl_otherwise(child_body: proc_macro2::TokenStream, body: &mut proc_macro2::TokenStream, methods: &mut proc_macro2::TokenStream, block_name: &str, format_char: char, ignore: &mut Vec<String>) {
     *body = quote!(
                         #body
                         sql.push_str(" ");
@@ -563,7 +549,7 @@ fn impl_otherwise(x: &Element, body: &mut proc_macro2::TokenStream, methods: &mu
 }
 
 
-fn impl_trim(prefix: &str, suffix: &str, prefixOverrides: &str, suffixOverrides: &str, x: &Element, body: &mut proc_macro2::TokenStream, arg: &Vec<Element>, methods: &mut proc_macro2::TokenStream, block_name: &str, format_char: char, ignore: &[&str]) {
+fn impl_trim(prefix: &str, suffix: &str, prefixOverrides: &str, suffixOverrides: &str, x: &Element, body: &mut proc_macro2::TokenStream, arg: &Vec<Element>, methods: &mut proc_macro2::TokenStream, block_name: &str, format_char: char, ignore: &mut Vec<String>) {
     let trim_body = parse(&x.childs, methods, block_name, format_char, ignore);
     let prefixs: Vec<&str> = prefixOverrides.split("|").collect();
     let suffixs: Vec<&str> = suffixOverrides.split("|").collect();
@@ -627,7 +613,7 @@ pub fn impl_fn_html(m: &ItemFn, args: &AttributeArgs) -> TokenStream {
             }
         };
     }
-    t = parse_html_str(&data, format_char, &fn_name, &[]);
+    t = parse_html_str(&data, format_char, &fn_name, &mut vec![]);
     return t.into();
 }
 
@@ -657,7 +643,7 @@ pub fn impl_fn_py(m: &ItemFn, args: &AttributeArgs) -> TokenStream {
         {
             println!("html:{}", htmls);
         }
-    t = parse_html_str(&htmls, format_char, &fn_name, &[]);
+    t = parse_html_str(&htmls, format_char, &fn_name, &mut vec![]);
     return t.into();
 }
 
